@@ -1,5 +1,5 @@
 #  Pyrogram - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of Pyrogram.
 #
@@ -18,14 +18,16 @@
 
 import logging
 
+import pyrogram
 from pyrogram import raw
-from pyrogram.scaffold import Scaffold
 
 log = logging.getLogger(__name__)
 
 
-class LogOut(Scaffold):
-    async def log_out(self):
+class LogOut:
+    async def log_out(
+        self: "pyrogram.Client",
+    ):
         """Log out from Telegram and delete the *\\*.session* file.
 
         When you log out, the current client is stopped and the storage session deleted.
@@ -40,7 +42,7 @@ class LogOut(Scaffold):
                 # Log out.
                 app.log_out()
         """
-        await self.send(raw.functions.auth.LogOut())
+        await self.invoke(raw.functions.auth.LogOut())
         await self.stop()
         await self.storage.delete()
 
