@@ -1,5 +1,5 @@
 #  Pyrogram - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2021 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of Pyrogram.
 #
@@ -18,14 +18,14 @@
 
 from typing import Union
 
+import pyrogram
 from pyrogram import raw
 from pyrogram import types
-from pyrogram.scaffold import Scaffold
 
 
-class RetractVote(Scaffold):
+class RetractVote:
     async def retract_vote(
-        self,
+        self: "pyrogram.Client",
         chat_id: Union[int, str],
         message_id: int
     ) -> "types.Poll":
@@ -46,9 +46,9 @@ class RetractVote(Scaffold):
         Example:
             .. code-block:: python
 
-                app.retract_vote(chat_id, message_id)
+                await app.retract_vote(chat_id, message_id)
         """
-        r = await self.send(
+        r = await self.invoke(
             raw.functions.messages.SendVote(
                 peer=await self.resolve_peer(chat_id),
                 msg_id=message_id,
