@@ -83,6 +83,7 @@ class Session:
         self.stored_msg_ids = []
 
         self.ping_task = None
+        self.ping_task_event = asyncio.Event()
 
         self.network_task = None
 
@@ -105,14 +106,14 @@ class Session:
 
                 self.network_task = self.loop.create_task(self.network_worker())
 
-                await self._send(raw.functions.Ping(ping_id=0), timeout=self.START_TIMEOUT)
+                await self.send(raw.functions.Ping(ping_id=0), timeout=self.START_TIMEOUT)
 
                 if not self.is_cdn:
-                    await self._send(
+                    await self.send(
                         raw.functions.InvokeWithLayer(
                             layer=layer,
                             query=raw.functions.InitConnection(
-                                api_id=self.client.api_id,
+                                api_id=await self.client.storage.api_id(),
                                 app_version=self.client.app_version,
                                 device_model=self.client.device_model,
                                 system_version=self.client.system_version,
@@ -149,23 +150,17 @@ class Session:
     async def stop(self):
         self.is_connected.clear()
 
-        if self.ping_task:
-            self.ping_task.cancel()
+        self.ping_task_event.set()
 
-            try:
-                await self.ping_task
-            except asyncio.CancelledError:
-                pass
+        if self.ping_task is not None:
+            await self.ping_task
+
+        self.ping_task_event.clear()
+
+        self.connection.close()
 
         if self.network_task:
-            self.network_task.cancel()
-
-            try:
-                await self.network_task
-            except asyncio.CancelledError:
-                pass
-
-        await self.connection.close()
+            await self.network_task
 
         for i in self.results.values():
             i.event.set()
@@ -194,7 +189,7 @@ class Session:
                 self.stored_msg_ids
             )
         except SecurityCheckMismatch:
-            await self.connection.close()
+            self.connection.close()
             return
 
         messages = (
@@ -245,61 +240,54 @@ class Session:
             log.debug(f"Send {len(self.pending_acks)} acks")
 
             try:
-                await self._send(raw.types.MsgsAck(msg_ids=list(self.pending_acks)), False)
+                await self.send(raw.types.MsgsAck(msg_ids=list(self.pending_acks)), False)
             except (OSError, TimeoutError):
                 pass
             else:
                 self.pending_acks.clear()
 
     async def ping_worker(self):
+        log.info("PingTask started")
+
         while True:
             try:
-                await self._send(
+                await asyncio.wait_for(self.ping_task_event.wait(), self.PING_INTERVAL)
+            except asyncio.TimeoutError:
+                pass
+            else:
+                break
+
+            try:
+                await self.send(
                     raw.functions.PingDelayDisconnect(
-                        ping_id=0,
-                        disconnect_delay=self.WAIT_TIMEOUT + 10
+                        ping_id=0, disconnect_delay=self.WAIT_TIMEOUT + 10
                     ), False
                 )
             except (OSError, TimeoutError, RPCError):
                 pass
 
-            await asyncio.sleep(self.PING_INTERVAL)
+        log.info("PingTask stopped")
 
     async def network_worker(self):
+        log.info("NetworkTask started")
+
         while True:
             packet = await self.connection.recv()
 
-            if not packet:
-                await self.connection.reconnect()
+            if packet is None or len(packet) == 4:
+                if packet:
+                    log.warning(f'Server sent "{Int.read(BytesIO(packet))}"')
 
-                try:
-                    await self._send(
-                        raw.functions.InvokeWithLayer(
-                            layer=layer,
-                            query=raw.functions.InitConnection(
-                                api_id=self.client.api_id,
-                                app_version=self.client.app_version,
-                                device_model=self.client.device_model,
-                                system_version=self.client.system_version,
-                                system_lang_code=self.client.lang_code,
-                                lang_code=self.client.lang_code,
-                                lang_pack="",
-                                query=raw.functions.help.GetConfig(),
-                            )
-                        ),
-                        wait_response=False
-                    )
-                except (OSError, TimeoutError, RPCError):
-                    pass
+                if self.is_connected.is_set():
+                    self.loop.create_task(self.restart())
 
-                continue
-
-            if len(packet) == 4:
-                log.warning(f'Server sent "{Int.read(BytesIO(packet))}"')
+                break
 
             self.loop.create_task(self.handle_packet(packet))
 
-    async def _send(self, data: TLObject, wait_response: bool = True, timeout: float = WAIT_TIMEOUT):
+        log.info("NetworkTask stopped")
+
+    async def send(self, data: TLObject, wait_response: bool = True, timeout: float = WAIT_TIMEOUT):
         message = self.msg_factory(data)
         msg_id = message.msg_id
 
@@ -346,13 +334,13 @@ class Session:
                 raise BadMsgNotification(result.error_code)
             elif isinstance(result, raw.types.BadServerSalt):
                 self.salt = result.new_server_salt
-                return await self._send(data, wait_response, timeout)
+                return await self.send(data, wait_response, timeout)
             else:
                 return result
 
-    async def send(
+    async def invoke(
         self,
-        data: TLObject,
+        query: TLObject,
         retries: int = MAX_RETRIES,
         timeout: float = WAIT_TIMEOUT,
         sleep_threshold: float = SLEEP_THRESHOLD
@@ -362,24 +350,22 @@ class Session:
         except asyncio.TimeoutError:
             pass
 
-        if isinstance(data, (raw.functions.InvokeWithoutUpdates, raw.functions.InvokeWithTakeout)):
-            query = data.query
-        else:
-            query = data
+        if isinstance(query, (raw.functions.InvokeWithoutUpdates, raw.functions.InvokeWithTakeout)):
+            query = query.query
 
-        query = ".".join(query.QUALNAME.split(".")[1:])
+        query_name = ".".join(query.QUALNAME.split(".")[1:])
 
         while True:
             try:
-                return await self._send(data, timeout=timeout)
+                return await self.send(query, timeout=timeout)
             except FloodWait as e:
-                amount = e.x
+                amount = e.value
 
                 if amount > sleep_threshold >= 0:
                     raise
 
-                log.warning(f'[{self.client.session_name}] Waiting for {amount} seconds before continuing '
-                            f'(required by "{query}")')
+                log.warning(f'[{self.client.name}] Waiting for {amount} seconds before continuing '
+                            f'(required by "{query_name}")')
 
                 await asyncio.sleep(amount)
             except (OSError, TimeoutError, InternalServerError, ServiceUnavailable) as e:
@@ -387,8 +373,8 @@ class Session:
                     raise e from None
 
                 (log.warning if retries < 2 else log.info)(
-                    f'[{Session.MAX_RETRIES - retries + 1}] Retrying "{query}" due to {str(e) or repr(e)}')
+                    f'[{Session.MAX_RETRIES - retries + 1}] Retrying "{query_name}" due to {str(e) or repr(e)}')
 
                 await asyncio.sleep(0.5)
 
-                return await self.send(data, retries - 1, timeout)
+                return await self.invoke(query, retries - 1, timeout)
