@@ -16,10 +16,11 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.
 
-from typing import Union, List, Generator, Optional
+from datetime import datetime
+from typing import Union, List
 
 import pyrogram
-from pyrogram import raw
+from pyrogram import raw, enums
 from pyrogram import types
 from pyrogram import utils
 from ..object import Object
@@ -32,8 +33,8 @@ class Chat(Object):
         id (``int``):
             Unique identifier for this chat.
 
-        type (``str``):
-            Type of chat, can be either "private", "bot", "group", "supergroup" or "channel".
+        type (:obj:`~pyrogram.enums.ChatType`):
+            Type of chat.
 
         is_verified (``bool``, *optional*):
             True, if this chat has been verified by Telegram. Supergroups, channels and bots only.
@@ -135,7 +136,7 @@ class Chat(Object):
         *,
         client: "pyrogram.Client" = None,
         id: int,
-        type: str,
+        type: "enums.ChatType",
         is_verified: bool = None,
         is_restricted: bool = None,
         is_creator: bool = None,
@@ -200,7 +201,7 @@ class Chat(Object):
 
         return Chat(
             id=peer_id,
-            type="bot" if user.bot else "private",
+            type=enums.ChatType.BOT if user.bot else enums.ChatType.PRIVATE,
             is_verified=getattr(user, "verified", None),
             is_restricted=getattr(user, "restricted", None),
             is_scam=getattr(user, "scam", None),
@@ -221,7 +222,7 @@ class Chat(Object):
 
         return Chat(
             id=peer_id,
-            type="group",
+            type=enums.ChatType.GROUP,
             title=chat.title,
             is_creator=getattr(chat, "creator", None),
             photo=types.ChatPhoto._parse(client, getattr(chat, "photo", None), peer_id, 0),
@@ -239,7 +240,7 @@ class Chat(Object):
 
         return Chat(
             id=peer_id,
-            type="supergroup" if getattr(channel, "megagroup", None) else "channel",
+            type=enums.ChatType.SUPERGROUP if getattr(channel, "megagroup", None) else enums.ChatType.CHANNEL,
             is_verified=getattr(channel, "verified", None),
             is_restricted=getattr(channel, "restricted", None),
             is_creator=getattr(channel, "creator", None),
@@ -364,12 +365,12 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.archive_chats(-100123456789)
+            await client.archive_chats(-100123456789)
 
         Example:
             .. code-block:: python
 
-                chat.archive()
+                await chat.archive()
 
         Returns:
             True on success.
@@ -387,12 +388,12 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.unarchive_chats(-100123456789)
+            await client.unarchive_chats(-100123456789)
 
         Example:
             .. code-block:: python
 
-                chat.unarchive()
+                await chat.unarchive()
 
         Returns:
             True on success.
@@ -411,7 +412,7 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.set_chat_title(
+            await client.set_chat_title(
                 chat_id=chat_id,
                 title=title
             )
@@ -419,7 +420,7 @@ class Chat(Object):
         Example:
             .. code-block:: python
 
-                chat.set_title("Lounge")
+                await chat.set_title("Lounge")
 
         Note:
             In regular groups (non-supergroups), this method will only work if the "All Members Are Admins"
@@ -449,7 +450,7 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.set_chat_description(
+            await client.set_chat_description(
                 chat_id=chat_id,
                 description=description
             )
@@ -457,7 +458,7 @@ class Chat(Object):
         Example:
             .. code-block:: python
 
-                chat.set_chat_description("Don't spam!")
+                await chat.set_chat_description("Don't spam!")
 
         Parameters:
             description (``str``):
@@ -483,7 +484,7 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.set_chat_photo(
+            await client.set_chat_photo(
                 chat_id=chat_id,
                 photo=photo
             )
@@ -491,7 +492,7 @@ class Chat(Object):
         Example:
             .. code-block:: python
 
-                chat.set_photo("photo.png")
+                await chat.set_photo("photo.png")
 
         Parameters:
             photo (``str``):
@@ -513,7 +514,7 @@ class Chat(Object):
     async def ban_member(
         self,
         user_id: Union[int, str],
-        until_date: int = 0
+        until_date: datetime = datetime.fromtimestamp(0)
     ) -> Union["types.Message", bool]:
         """Bound method *ban_member* of :obj:`~pyrogram.types.Chat`.
 
@@ -521,7 +522,7 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.ban_chat_member(
+            await client.ban_chat_member(
                 chat_id=chat_id,
                 user_id=user_id
             )
@@ -529,7 +530,7 @@ class Chat(Object):
         Example:
             .. code-block:: python
 
-                chat.ban_member(123456789)
+                await chat.ban_member(123456789)
 
         Note:
             In regular groups (non-supergroups), this method will only work if the "All Members Are Admins" setting is
@@ -541,10 +542,10 @@ class Chat(Object):
                 Unique identifier (int) or username (str) of the target user.
                 For a contact that exists in your Telegram address book you can use his phone number (str).
 
-            until_date (``int``, *optional*):
-                Date when the user will be unbanned, unix time.
+            until_date (:py:obj:`~datetime.datetime`, *optional*):
+                Date when the user will be unbanned.
                 If user is banned for more than 366 days or less than 30 seconds from the current time they are
-                considered to be banned forever. Defaults to 0 (ban forever).
+                considered to be banned forever. Defaults to epoch (ban forever).
 
         Returns:
             :obj:`~pyrogram.types.Message` | ``bool``: On success, a service message will be returned (when applicable), otherwise, in
@@ -570,7 +571,7 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.unban_chat_member(
+            await client.unban_chat_member(
                 chat_id=chat_id,
                 user_id=user_id
             )
@@ -578,7 +579,7 @@ class Chat(Object):
         Example:
             .. code-block:: python
 
-                chat.unban_member(123456789)
+                await chat.unban_member(123456789)
 
         Parameters:
             user_id (``int`` | ``str``):
@@ -601,7 +602,7 @@ class Chat(Object):
         self,
         user_id: Union[int, str],
         permissions: "types.ChatPermissions",
-        until_date: int = 0,
+        until_date: datetime = datetime.fromtimestamp(0),
     ) -> "types.Chat":
         """Bound method *unban_member* of :obj:`~pyrogram.types.Chat`.
 
@@ -609,7 +610,7 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.restrict_chat_member(
+            await client.restrict_chat_member(
                 chat_id=chat_id,
                 user_id=user_id,
                 permissions=ChatPermissions()
@@ -618,7 +619,7 @@ class Chat(Object):
         Example:
             .. code-block:: python
 
-                chat.restrict_member(user_id, ChatPermissions())
+                await chat.restrict_member(user_id, ChatPermissions())
 
         Parameters:
             user_id (``int`` | ``str``):
@@ -628,10 +629,10 @@ class Chat(Object):
             permissions (:obj:`~pyrogram.types.ChatPermissions`):
                 New user permissions.
 
-            until_date (``int``, *optional*):
-                Date when the user will be unbanned, unix time.
+            until_date (:py:obj:`~datetime.datetime`, *optional*):
+                Date when the user will be unbanned.
                 If user is banned for more than 366 days or less than 30 seconds from the current time they are
-                considered to be banned forever. Defaults to 0 (ban forever).
+                considered to be banned forever. Defaults to epoch (ban forever).
 
         Returns:
             :obj:`~pyrogram.types.Chat`: On success, a chat object is returned.
@@ -647,19 +648,12 @@ class Chat(Object):
             until_date=until_date,
         )
 
+    # Set None as privileges default due to issues with partially initialized module, because at the time Chat
+    # is being initialized, ChatPrivileges would be required here, but was not initialized yet.
     async def promote_member(
         self,
         user_id: Union[int, str],
-        can_manage_chat: bool = True,
-        can_change_info: bool = True,
-        can_post_messages: bool = False,
-        can_edit_messages: bool = False,
-        can_delete_messages: bool = True,
-        can_restrict_members: bool = True,
-        can_invite_users: bool = True,
-        can_pin_messages: bool = False,
-        can_promote_members: bool = False,
-        can_manage_voice_chats: bool = False
+        privileges: "types.ChatPrivileges" = None
     ) -> bool:
         """Bound method *promote_member* of :obj:`~pyrogram.types.Chat`.
 
@@ -667,7 +661,7 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.promote_chat_member(
+            await client.promote_chat_member(
                 chat_id=chat_id,
                 user_id=user_id
             )
@@ -676,46 +670,15 @@ class Chat(Object):
 
             .. code-block:: python
 
-                chat.promote_member(123456789)
+                await chat.promote_member(123456789)
 
         Parameters:
             user_id (``int`` | ``str``):
                 Unique identifier (int) or username (str) of the target user.
                 For a contact that exists in your Telegram address book you can use his phone number (str).
 
-            can_manage_chat (``bool``, *optional*):
-                Pass True, if the administrator can access the chat event log, chat statistics, message statistics
-                in channels, see channel members, see anonymous administrators in supergroups and ignore slow mode.
-                Implied by any other administrator privilege.
-
-            can_change_info (``bool``, *optional*):
-                Pass True, if the administrator can change chat title, photo and other settings.
-
-            can_post_messages (``bool``, *optional*):
-                Pass True, if the administrator can create channel posts, channels only.
-
-            can_edit_messages (``bool``, *optional*):
-                Pass True, if the administrator can edit messages of other users and can pin messages, channels only.
-
-            can_delete_messages (``bool``, *optional*):
-                Pass True, if the administrator can delete messages of other users.
-
-            can_restrict_members (``bool``, *optional*):
-                Pass True, if the administrator can restrict, ban or unban chat members.
-
-            can_invite_users (``bool``, *optional*):
-                Pass True, if the administrator can invite new users to the chat.
-
-            can_pin_messages (``bool``, *optional*):
-                Pass True, if the administrator can pin messages, supergroups only.
-
-            can_promote_members (``bool``, *optional*):
-                Pass True, if the administrator can add new administrators with a subset of his own privileges or
-                demote administrators that he has promoted, directly or indirectly (promoted by administrators that
-                were appointed by him).
-
-            can_manage_voice_chats (``bool``, *optional*):
-                Pass True, if the administration can manage voice chats (also called group calls).
+            privileges (:obj:`~pyrogram.types.ChatPrivileges`, *optional*):
+                New user privileges.
 
         Returns:
             ``bool``: True on success.
@@ -727,16 +690,7 @@ class Chat(Object):
         return await self._client.promote_chat_member(
             chat_id=self.id,
             user_id=user_id,
-            can_manage_chat=can_manage_chat,
-            can_change_info=can_change_info,
-            can_post_messages=can_post_messages,
-            can_edit_messages=can_edit_messages,
-            can_delete_messages=can_delete_messages,
-            can_restrict_members=can_restrict_members,
-            can_invite_users=can_invite_users,
-            can_pin_messages=can_pin_messages,
-            can_promote_members=can_promote_members,
-            can_manage_voice_chats=can_manage_voice_chats
+            privileges=privileges
         )
 
     async def join(self):
@@ -746,12 +700,12 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.join_chat(123456789)
+            await client.join_chat(123456789)
 
         Example:
             .. code-block:: python
 
-                chat.join()
+                await chat.join()
 
         Note:
             This only works for public groups, channels that have set a username or linked chats.
@@ -772,12 +726,12 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.leave_chat(123456789)
+            await client.leave_chat(123456789)
 
         Example:
             .. code-block:: python
 
-                chat.leave()
+                await chat.leave()
 
         Raises:
             RPCError: In case of a Telegram RPC error.
@@ -818,7 +772,7 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.get_chat_member(
+            await client.get_chat_member(
                 chat_id=chat_id,
                 user_id=user_id
             )
@@ -826,7 +780,7 @@ class Chat(Object):
         Example:
             .. code-block:: python
 
-                chat.get_member(user_id)
+                await chat.get_member(user_id)
 
         Returns:
             :obj:`~pyrogram.types.ChatMember`: On success, a chat member is returned.
@@ -839,10 +793,9 @@ class Chat(Object):
 
     async def get_members(
         self,
-        offset: int = 0,
-        limit: int = 200,
         query: str = "",
-        filter: str = "all"
+        limit: int = 0,
+        filter: "enums.ChatMembersFilter" = enums.ChatMembersFilter.SEARCH
     ) -> List["types.ChatMember"]:
         """Bound method *get_members* of :obj:`~pyrogram.types.Chat`.
 
@@ -850,126 +803,38 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.get_chat_members(chat_id)
-
-
-        Parameters:
-            offset (``int``, *optional*):
-                Sequential number of the first member to be returned.
-                Only applicable to supergroups and channels. Defaults to 0 [1]_.
-
-            limit (``int``, *optional*):
-                Limits the number of members to be retrieved.
-                Only applicable to supergroups and channels.
-                Defaults to 200, which is also the maximum server limit allowed per method call.
-
-            query (``str``, *optional*):
-                Query string to filter members based on their display names and usernames.
-                Only applicable to supergroups and channels. Defaults to "" (empty string) [2]_.
-
-            filter (``str``, *optional*):
-                Filter used to select the kind of members you want to retrieve. Only applicable for supergroups
-                and channels. It can be any of the followings:
-                *"all"* - all kind of members,
-                *"banned"* - banned members only,
-                *"restricted"* - restricted members only,
-                *"bots"* - bots only,
-                *"recent"* - recent members only,
-                *"administrators"* - chat administrators only.
-                Only applicable to supergroups and channels.
-                Defaults to *"recent"*.
-
-        .. [1] Server limit: on supergroups, you can get up to 10,000 members for a single query and up to 200 members
-            on channels.
-
-        .. [2] A query string is applicable only for *"all"*, *"banned"* and *"restricted"* filters only.
+            async for member in client.get_chat_members(chat_id):
+                print(member)
 
         Example:
             .. code-block:: python
 
-                # Get first 200 recent members
-                chat.get_members()
-
-                # Get all administrators
-                chat.get_members(filter="administrators")
-
-                # Get all bots
-                chat.get_members(filter="bots")
-
-        Returns:
-            List of :obj:`~pyrogram.types.ChatMember`: On success, a list of chat members is returned.
-        """
-
-        return await self._client.get_chat_members(
-            self.id,
-            offset=offset,
-            limit=limit,
-            query=query,
-            filter=filter
-        )
-
-    def iter_members(
-        self,
-        limit: int = 0,
-        query: str = "",
-        filter: str = "all"
-    ) -> Optional[Generator["types.ChatMember", None, None]]:
-        """Bound method *iter_members* of :obj:`~pyrogram.types.Chat`.
-
-        Use as a shortcut for:
-
-        .. code-block:: python
+                async for member in chat.get_members():
+                    print(member)
 
         Parameters:
-            limit (``int``, *optional*):
-                Limits the number of members to be retrieved.
-                Only applicable to supergroups and channels.
-                Defaults to 200, which is also the maximum server limit allowed per method call [1]_.
-
             query (``str``, *optional*):
                 Query string to filter members based on their display names and usernames.
-                Only applicable to supergroups and channels. Defaults to "" (empty string) [2]_.
+                Only applicable to supergroups and channels. Defaults to "" (empty string).
+                A query string is applicable only for :obj:`~pyrogram.enums.ChatMembersFilter.SEARCH`,
+                :obj:`~pyrogram.enums.ChatMembersFilter.BANNED` and :obj:`~pyrogram.enums.ChatMembersFilter.RESTRICTED`
+                filters only.
 
-            filter (``str``, *optional*):
+            limit (``int``, *optional*):
+                Limits the number of members to be retrieved.
+
+            filter (:obj:`~pyrogram.enums.ChatMembersFilter`, *optional*):
                 Filter used to select the kind of members you want to retrieve. Only applicable for supergroups
-                and channels. It can be any of the followings:
-                *"all"* - all kind of members,
-                *"banned"* - banned members only,
-                *"restricted"* - restricted members only,
-                *"bots"* - bots only,
-                *"recent"* - recent members only,
-                *"administrators"* - chat administrators only.
-                Only applicable to supergroups and channels.
-                Defaults to *"recent"*.
-
-        .. [1] Server limit: on supergroups, you can get up to 10,000 members for a single query and up to 200 members
-            on channels.
-
-        .. [2] A query string is applicable only for *"all"*, *"banned"* and *"restricted"* filters only.
-
-        Example:
-            .. code-block:: python
-
-                # Get first 200 recent members
-                for member in chat.get_members():
-                    print(member.user.first_name)
-
-                # Get all administrators
-                for member in chat.iter_members(filter="administrators"):
-                    print(member.user.first_name)
-
-                # Get first 3 bots
-                for member in chat.iter_members(filter="bots", limit=3):
-                    print(member.user.first_name)
+                and channels.
 
         Returns:
-            ``Generator``: A generator yielding :obj:`~pyrogram.types.ChatMember` objects.
+            ``Generator``: On success, a generator yielding :obj:`~pyrogram.types.ChatMember` objects is returned.
         """
 
-        return self._client.iter_chat_members(
+        return self._client.get_chat_members(
             self.id,
-            limit=limit,
             query=query,
+            limit=limit,
             filter=filter
         )
 
@@ -984,12 +849,12 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.add_chat_members(chat_id, user_id)
+            await client.add_chat_members(chat_id, user_id)
 
         Example:
             .. code-block:: python
 
-                chat.add_members(user_id)
+                await chat.add_members(user_id)
 
         Returns:
             ``bool``: On success, True is returned.
@@ -1008,12 +873,12 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.mark_unread(chat_id)
+            await client.mark_unread(chat_id)
 
         Example:
             .. code-block:: python
 
-                chat.mark_unread()
+                await chat.mark_unread()
 
         Returns:
             ``bool``: On success, True is returned.
@@ -1028,7 +893,7 @@ class Chat(Object):
 
         .. code-block:: python
 
-            client.set_chat_protected_content(chat_id, enabled)
+            await client.set_chat_protected_content(chat_id, enabled)
 
         Parameters:
             enabled (``bool``):
@@ -1037,7 +902,7 @@ class Chat(Object):
         Example:
             .. code-block:: python
 
-                chat.set_protected_content(enabled)
+                await chat.set_protected_content(enabled)
 
         Returns:
             ``bool``: On success, True is returned.
